@@ -15,6 +15,17 @@ import imagej
 # JPype
 from jpype.types import JString, JArray
 
+def get_java_dependencies():
+    """
+    Returns the jar files that need to be included into the classpath
+    of an imagej object in order to have a functional ABBA app
+    these jars should be available in https://maven.scijava.org/
+    :return:
+    """
+    imagej_core_dep = 'net.imagej:imagej:2.14.0'
+    imagej_legacy_dep = 'net.imagej:imagej-legacy:1.2.1'
+    abba_dep = 'ch.epfl.biop:ImageToAtlasRegister:0.8.0'
+    return [imagej_core_dep, imagej_legacy_dep, abba_dep]
 
 def start_imagej(headless: bool = False):
     mode = "headless"
@@ -58,20 +69,6 @@ def start_imagej(headless: bool = False):
     if not headless:
         ij.ui().showUI()
 
-
-def get_java_dependencies():
-    """
-    Returns the jar files that need to be included into the classpath
-    of an imagej object in order to have a functional ABBA app
-    these jars should be available in https://maven.scijava.org/
-    :return:
-    """
-    imagej_core_dep = 'net.imagej:imagej:2.14.0'
-    imagej_legacy_dep = 'net.imagej:imagej-legacy:1.2.1'
-    abba_dep = 'ch.epfl.biop:ImageToAtlasRegister:0.7.2'
-    return [imagej_core_dep, imagej_legacy_dep, abba_dep]
-
-
 def add_brainglobe_atlases(ij):
     # TODO : check connection available or not
     try:
@@ -85,14 +82,6 @@ def add_brainglobe_atlases(ij):
             available_atlases[atlas] = get_local_atlas_version(atlas)
 
     AtlasChooserCommand = jimport('ch.epfl.biop.atlas.scijava.AtlasChooserCommand')
-    try:
-        from abba_python.abba_private.AbbaAtlas import AbbaAtlas  # delayed import because the jvm should be set first
-    except:
-        try:
-            from .abba_private.AbbaAtlas import \
-                AbbaAtlas  # delayed import because the jvm should be set first
-        except:
-            print("Could not import AbbaAtlas because the dev do not understand python modules and submodules.")
 
     from jpype import JImplements, JOverride
     Supplier = jimport('java.util.function.Supplier')
@@ -109,6 +98,7 @@ def add_brainglobe_atlases(ij):
         @JOverride
         def get(self):
             bg_atlas = BrainGlobeAtlas(self.atlas_name)
+            from abba_python.abba_private.abba_atlas import AbbaAtlas  # but wth ????
             current_atlas = AbbaAtlas(bg_atlas, self.ij)
             current_atlas.initialize(None, None)
             Abba.opened_atlases[self.atlas_name] = current_atlas
@@ -187,19 +177,9 @@ class Abba:
                 Abba.opened_atlases[atlas_name] = atlas
             else:
                 bg_atlas = BrainGlobeAtlas(atlas_name)
-
-                try:
-                    from abba_python.abba_private.AbbaAtlas import \
-                        AbbaAtlas  # delayed import because the jvm should be set first
-                except:
-                    try:
-                        from .abba_private.AbbaAtlas import \
-                            AbbaAtlas  # delayed import because the jvm should be set first
-                    except:
-                        print("Could not import AbbaAtlas")
-
                 # initialized
-                atlas = AbbaAtlas(bg_atlas, ij)
+                from abba_private import abba_atlas
+                atlas = abba_atlas.AbbaAtlas(bg_atlas, ij)
                 atlas.initialize(None, None)
                 Abba.opened_atlases[atlas_name] = atlas
                 ij.object().addObject(atlas, atlas_name)  # store it in java's object service
@@ -687,6 +667,22 @@ class Abba:
                                      'image', image,
                                      'slice_axis_mm', slice_axis_mm).get()
 
+    def import_slice_from_sources(self,
+                                  slice_axis_mm: float,
+                                  sources):
+        """
+        Import a list of sources as a slice into ABBA
+
+        Parameters:
+        slice_axis_mm (float): Initial axis position (0 = front, mm units)
+        sources :
+        """
+        ImportSliceFromSourcesCommand = jimport('ch.epfl.biop.atlas.aligner.command.ImportSliceFromSourcesCommand')
+        return self.ij.command().run(ImportSliceFromSourcesCommand, True,
+                                     'mp', self.mp,
+                                     'slice_axis_mm', slice_axis_mm,
+                                     'sources', sources).get()
+
     def import_slices_from_files(self,
                                  datasetname: str,
                                  files,
@@ -798,36 +794,74 @@ class Abba:
                                      'channel_atlas', channel_atlas,
                                      'channel_slice', channel_slice).get()
 
-    def register_slices_deepslice(self,
-                                  affine_transform: bool,
-                                  allow_change_slicing_position: bool,
-                                  allow_slicing_angle_change: bool,
-                                  channels: str,
-                                  maintain_slices_order: bool,
-                                  model: str,
-                                  run_mode: str):
+    def register_slices_copy_and_apply(self,
+                                       model_slice_index: int,
+                                       skip_pre_transform: bool):
         """
-        Uses Deepslice Web interface for affine in plane and axial registration of selected slices
+        Copy the registration sequence of a slice and apply it to selected slices
 
         Parameters:
-        affine_transform (bool): Affine transform in plane
-        allow_change_slicing_position (bool): Allow change of position along the slicing axis
-        allow_slicing_angle_change (bool): Allow change of atlas slicing angle
-        channels : Slices channels, 0-based
-        maintain_slices_order (bool): Maintain the rank of the slices
-        model (str): ('mouse', 'rat') Mouse or Rat ?
-        run_mode (str): ('local', 'web') Local Conda Env or Web ?
+        model_slice_index (int): Index of the slice registrations you'd like to copy
+        skip_pre_transform (bool): Tick if you want to skip the pre-transform (probably not)
         """
-        RegisterSlicesDeepSliceCommand = jimport('ch.epfl.biop.atlas.aligner.command.RegisterSlicesDeepSliceCommand')
-        return self.ij.command().run(RegisterSlicesDeepSliceCommand, True,
+        RegisterSlicesCopyAndApplyCommand = jimport(
+            'ch.epfl.biop.atlas.aligner.command.RegisterSlicesCopyAndApplyCommand')
+        return self.ij.command().run(RegisterSlicesCopyAndApplyCommand, True,
+                                     'mp', self.mp,
+                                     'model_slice_index', model_slice_index,
+                                     'skip_pre_transform', skip_pre_transform).get()
+
+    def register_slices_deepslice_local(self,
+                                        allow_slicing_angle_change: bool,
+                                        channels: str,
+                                        ensemble: bool,
+                                        model: str,
+                                        post_processing: str,
+                                        slices_spacing_micrometer: float):
+        """
+        Uses Deepslice for affine in plane and axial registration of selected slices
+
+        Parameters:
+        allow_slicing_angle_change (bool): Allow change of atlas slicing angle
+        channels (str): Slices channels, 0-based, comma separated, '*' for all channels
+        ensemble (bool): Average of several models (slower)
+        model (str): ('mouse', 'rat') Mouse or Rat ?
+        post_processing (str):
+        slices_spacing_micrometer (float): Spacing (micrometer), used only when 'Keep order + set spacing' is selected
+        """
+        RegisterSlicesDeepSliceLocalCommand = jimport(
+            'ch.epfl.biop.atlas.aligner.command.RegisterSlicesDeepSliceLocalCommand')
+        return self.ij.command().run(RegisterSlicesDeepSliceLocalCommand, True,
                                      'mp', self.mp,
-                                     'affine_transform', affine_transform,
-                                     'allow_change_slicing_position', allow_change_slicing_position,
                                      'allow_slicing_angle_change', allow_slicing_angle_change,
-                                     'channels', JString(','.join(map(str, channels))),
+                                     'channels', channels,
+                                     'ensemble', ensemble,
+                                     'model', model,
+                                     'post_processing', post_processing,
+                                     'slices_spacing_micrometer', slices_spacing_micrometer).get()
+
+    def register_slices_deepslice_web(self,
+                                      allow_slicing_angle_change: bool,
+                                      channels: str,
+                                      maintain_slices_order: bool,
+                                      model: str):
+        """
+        Uses Deepslice for affine in plane and axial registration of selected slices
+
+        Parameters:
+        allow_slicing_angle_change (bool): Allow change of atlas slicing angle
+        channels (str): Slices channels, 0-based, comma separated, '*' for all channels
+        maintain_slices_order (bool): Keep slices order
+        model (str): ('mouse', 'rat') Mouse or Rat ?
+        """
+        RegisterSlicesDeepSliceWebCommand = jimport(
+            'ch.epfl.biop.atlas.aligner.command.RegisterSlicesDeepSliceWebCommand')
+        return self.ij.command().run(RegisterSlicesDeepSliceWebCommand, True,
+                                     'mp', self.mp,
+                                     'allow_slicing_angle_change', allow_slicing_angle_change,
+                                     'channels', channels,
                                      'maintain_slices_order', maintain_slices_order,
-                                     'model', JString(model),
-                                     'run_mode', JString(run_mode)).get()
+                                     'model', model).get()
 
     def register_slices_edit_last(self,
                                   atlas_channels_csv: str,
@@ -875,7 +909,6 @@ class Abba:
                                      'show_imageplus_registration_result', show_imageplus_registration_result).get()
 
     def register_slices_elastix_spline(self,
-
                                        channels_atlas_csv: str,
                                        channels_slice_csv: str,
                                        nb_control_points_x: int,
